@@ -18,7 +18,7 @@
  * each of which is an AG and has an ag_header at the beginning.
  */
 typedef struct ag_header  {
-	xfs_dsb_t	*xfs_sb;	/* superblock for filesystem or AG */
+	struct xfs_dsb	*xfs_sb;	/* superblock for filesystem or AG */
 	xfs_agf_t	*xfs_agf;	/* free space info */
 	xfs_agi_t	*xfs_agi;	/* free inode info */
 	struct xfs_agfl	*xfs_agfl;	/* AG freelist */
