@@ -81,6 +81,9 @@
 #define trace_xfs_iext_insert(a,b,c,d)		((void) 0)
 #define trace_xfs_iext_remove(a,b,c,d)		((void) 0)
 
+#define trace_xfs_defer_add_item(a,b,c)		((void) 0)
+#define trace_xfs_defer_cancel_item(a,b,c)	((void) 0)
+#define trace_xfs_defer_finish_item(a,b,c)	((void) 0)
 #define trace_xfs_defer_relog_intent(a,b)	((void) 0)
 
 #define trace_xfs_dir2_grow_inode(a,b)		((void) 0)
@@ -189,6 +192,7 @@
 #define trace_xfs_write_extent(a,b,c,d)	((c) = (c))
 #define trace_xfs_perag_get(c,d)		((c) = (c))
 #define trace_xfs_perag_get_tag(c,d)		((c) = (c))
+#define trace_xfs_perag_hold(...)		((void) 0)
 #define trace_xfs_perag_grab(...)		((void) 0)
 #define trace_xfs_perag_grab_tag(...)		((void) 0)
 #define trace_xfs_perag_put(c,d)		((c) = (c))
